package com.example.moviestudio.ui.components

import android.annotation.SuppressLint
import android.util.Log
import android.widget.Toast
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyRow
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.DisposableEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.unit.dp
import com.example.moviestudio.R
import com.example.moviestudio.ui.theme.second
import com.example.moviestudio.viewmodels.LibraryViewModel

@SuppressLint("UnusedMaterial3ScaffoldPaddingParameter")
@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun LibrarySearchBar(
    onMovieNameValueChange: (String) -> Unit,
    onMoviePublishYearValueChange: (String) -> Unit,
    onActorValueChange: (String) -> Unit,
    onMovieGenreValueChange: (String) -> Unit,
    onSearchClick: () -> Unit,
    onClearClick: () -> Unit,
    movieTitle: String,
    movieYearReleaseDate: String,
    movieGenres: String,
    movieActors: String
) {
    val context = LocalContext.current
    val toastText = stringResource(id = R.string.library_toaster_fitler_search)
    Row {
        Column {

            Button(
                colors = ButtonDefaults.buttonColors(containerColor = second),
                modifier = Modifier
                    .width(80.dp)
                    .padding(top = 75.dp)
                    .padding(start = 5.dp, end = 5.dp),
                onClick = {
                    onSearchClick()
                    Toast.makeText(context, toastText, Toast.LENGTH_SHORT).show()
                }) {
                Icon(
                    painterResource(id = R.drawable.baseline_search_24),
                    contentDescription = "Search",
                    tint = Color.White
                )
            }
            Button(
                colors = ButtonDefaults.buttonColors(containerColor = second),
                modifier = Modifier
                    .width(80.dp)
                    .padding(start = 5.dp, end = 5.dp),
                onClick = {
                    onClearClick()
                }) {
                Icon(
                    painterResource(id = R.drawable.baseline_clear_24),
                    contentDescription = "Clear",
                    tint = Color.White
                )
            }
        }
        LazyRow(
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = 15.dp)
        ) {
            item {
                MovieNameInput(onMovieNameValueChange, movieTitle, onSearchClick)
            }
            item {
                MovieReleaseYearInput(onMoviePublishYearValueChange, movieYearReleaseDate, onSearchClick)
            }
            item {
                ActorNameInput(onActorValueChange, movieActors, onSearchClick)
            }
            item {
                MovieGenreInput(onMovieGenreValueChange, movieGenres, onSearchClick)
            }
        }
    }

}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
internal fun MovieNameInput(
    onValueChange: (String) -> Unit,
    value: String,
    onSearchClick: () -> Unit
) {
    var text by remember { mutableStateOf(value) }
    var txtMovieName = stringResource(id = R.string.library_placeholder_movie_title)

    DisposableEffect(value) {
        text = value
        onValueChange(value)
        onDispose { }
    }

    OutlinedTextField(
        text,
        onValueChange = {
            text = it
            onValueChange(it)
        },
        label = { Text(txtMovieName) },
        modifier = Modifier
            .padding(top = 55.dp)
            .padding(8.dp),
        textStyle = TextStyle(color = Color.White),
        keyboardOptions = KeyboardOptions.Default.copy(
            imeAction = ImeAction.Done
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                onSearchClick()
            }
        )
    )
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
internal fun MovieReleaseYearInput(
    onValueChange: (String) -> Unit,
    value: String,
    onSearchClick: () -> Unit
) {
    var text by remember { mutableStateOf(value) }
    var txtReleaseDate = stringResource(id = R.string.library_placeholder_movie_year_release_date)

    DisposableEffect(value) {
        text = value
        onValueChange(value)
        onDispose { }
    }

    OutlinedTextField(
        text,
        onValueChange = {
            val newValue = it.filter { char -> char.isDigit() }
            text = newValue
            onValueChange(newValue)
        },
        label = { Text(txtReleaseDate) },
        modifier = Modifier
            .padding(8.dp)
            .padding(top = 55.dp),
        textStyle = TextStyle(color = Color.White),
        keyboardOptions = KeyboardOptions.Default.copy(
            imeAction = ImeAction.Done
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                onSearchClick()
            }
        )
    )
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
internal fun ActorNameInput(
    onValueChange: (String) -> Unit,
    value: String,
    onSearchClick: () -> Unit
) {
    var text by remember { mutableStateOf(value) }
    var txtActor = stringResource(id = R.string.library_placeholder_movie_actor)

    DisposableEffect(value) {
        text = value
        onValueChange(value)
        onDispose { }
    }

    OutlinedTextField(
        text,
        onValueChange = {
            text = it
            onValueChange(it)
        },
        label = { Text(txtActor) },
        modifier = Modifier
            .padding(8.dp)
            .padding(top = 55.dp),
        textStyle = TextStyle(color = Color.White),
        keyboardOptions = KeyboardOptions.Default.copy(
            imeAction = ImeAction.Done
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                onSearchClick()
            }
        )
    )
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
internal fun MovieGenreInput(
    onValueChange: (String) -> Unit,
    value: String,
    onSearchClick: () -> Unit
) {
    var text by remember { mutableStateOf(value) }
    var txtGenre = stringResource(id = R.string.library_placeholder_movie_genre)

    DisposableEffect(value) {
        text = value
        onValueChange(value)
        onDispose { }
    }

    OutlinedTextField(
        text,
        onValueChange = {
            text = it
            onValueChange(it)
        },
        label = { Text(txtGenre) },
        modifier = Modifier
            .padding(8.dp)
            .padding(top = 55.dp),
        textStyle = TextStyle(color = Color.White),
        keyboardOptions = KeyboardOptions.Default.copy(
            imeAction = ImeAction.Done
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                onSearchClick()
            }
        )
    )
}
