package com.example.moviestudio.ui.screens

import android.annotation.SuppressLint
import android.graphics.fonts.FontStyle
import android.util.Log
import android.widget.Toast
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Observer
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.NavController
import com.example.moviestudio.Movie
import com.example.moviestudio.R
import com.example.moviestudio.ui.components.BottomNavigation
import com.example.moviestudio.ui.components.MovieCardLibrary
import com.example.moviestudio.ui.components.LibrarySearchBar
import com.example.moviestudio.ui.components.ShowMovie
import com.example.moviestudio.ui.theme.dark
import com.example.moviestudio.ui.theme.main
import com.example.moviestudio.viewmodels.LibraryViewModel
import com.example.moviestudio.viewmodels.ShowMovieViewModel
import kotlinx.coroutines.launch
import androidx.compose.runtime.DisposableEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.mutableLongStateOf
import androidx.compose.ui.platform.LocalContext
import com.example.moviestudio.ui.components.AlertDialogSuppression
import kotlinx.coroutines.Dispatchers


@SuppressLint("UnusedMaterial3ScaffoldPaddingParameter")
@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun Library(navController: NavController) {
    val appName = stringResource(id = R.string.app_name)
    val pageName = stringResource(id = R.string.library_title)
    val noMoviesInLibrary = stringResource(id = R.string.library_empty)
    val toastText = stringResource(id = R.string.library_toaster_clear_filters_search)
    val toastTextDeleteMovie = stringResource(id = R.string.library_toaster_remove_movie)
    val context = LocalContext.current

    val viewModel = viewModel<LibraryViewModel>()
    val viewModelShow = viewModel<ShowMovieViewModel>()

    val movieFlow by viewModel.movieFlow.collectAsState()

    val txtModalTitle = stringResource(id = R.string.library_modal_suppression_title)
    val txtModalText = stringResource(id = R.string.library_modal_suppression_text)
    var isDialogVisible by remember { mutableStateOf(false) }
    var movieId by remember { mutableLongStateOf(0L) }

    Scaffold(
        modifier = Modifier,
        topBar = {
            if (!viewModel.isModalVisible) {
                TopAppBar(colors = TopAppBarDefaults.smallTopAppBarColors(
                    containerColor = main,
                    titleContentColor = Color.White,
                ), title = {
                    Column {
                        Text(text = appName, fontSize = 24.sp)
                        Text(text = pageName, fontSize = 16.sp)
                    }
                }
                )
            }
        },
        content = {
            // Content with a Column containing search components and LazyColumn
            Column(
                modifier = Modifier
                    .background(dark)
                    .fillMaxSize()
                    .padding(4.dp),
                horizontalAlignment = Alignment.CenterHorizontally
            ) {
                if (!viewModel.isModalVisible) {
                    LibrarySearchBar(
                        onMovieNameValueChange = { viewModel.updateSearchMovieName(it) },
                        onMoviePublishYearValueChange = { viewModel.updateSearchPublishYear(it) },
                        onActorValueChange = { viewModel.updateSearchActor(it) },
                        onMovieGenreValueChange = { viewModel.updateSearchGenre(it) },
                        onSearchClick = { viewModel.searchMoviesWithFilter() },
                        onClearClick = {
                            viewModel.clearInputs()
                            Toast.makeText(context, toastText, Toast.LENGTH_SHORT).show()
                            viewModel.searchMoviesWithFilter()
                        },
                        movieTitle = viewModel.searchQuery,
                        movieActors = viewModel.searchActor,
                        movieGenres = viewModel.searchGenre,
                        movieYearReleaseDate = viewModel.searchPublishDate
                    )
                }

                if (movieFlow.isEmpty()) {
                    Text(
                        text = noMoviesInLibrary,
                        color = Color.Gray,
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(8.dp)
                    )
                }

                // Display movies saved in the library.
                LazyColumn(
                    modifier = Modifier
                        .fillMaxWidth()
                        .weight(1f)
                ) {
                    items(movieFlow) { movie ->
                        MovieCardLibrary(
                            movie = movie,
                            onKnowMoreClick = {
                                viewModel.updateSelectedMovie(movie)
                                viewModel.showMovieDetails()
                                viewModelShow.fetchRating(movie.id)
                            },
                            onDeleteClick = {
                                isDialogVisible = true
                                movieId = movie.id
                            }
                        )
                    }
                }

                ShowMovie(
                    isModalVisible = viewModel.isModalVisible,
                    onCloseButtonClick = { viewModel.hideMovieDetails() },
                    movie = viewModel.selectedMovie
                )
                Spacer(modifier = Modifier.height(55.dp))
            }
        },
        bottomBar = {
            BottomNavigation(navController = navController)
        }
    )
    if (isDialogVisible) {
        AlertDialogSuppression(
            onDismissRequest = {
                isDialogVisible = false
            },
            onConfirmation = {
                viewModel.deleteMovie(movieId)
                Toast.makeText(context, toastTextDeleteMovie, Toast.LENGTH_SHORT).show()
                isDialogVisible = false
            },
            dialogTitle = txtModalTitle,
            dialogText = txtModalText
        )
    }
}
